@@ -15,11 +15,8 @@
     Contact: maxime.perrotin@esa.int
 """
 
-import os
-import sys
 import math
 import logging
-from collections import namedtuple
 
 from PySide.QtCore import Qt, QPointF, QLineF
 
@@ -27,6 +24,8 @@ from PySide.QtGui import QGraphicsPathItem, QPainterPath, QGraphicsItem, QPen,\
                          QPainter, QFont, QGraphicsTextItem, QColor, \
                          QFontMetrics
 
+LOG = logging.getLogger(__name__)
+
 
 # pylint: disable=R0904
 class Connection(QGraphicsPathItem, object):
@@ -40,7 +39,7 @@ class Connection(QGraphicsPathItem, object):
         super(Connection, self).__init__(parent)
         self.parent = parent
         self.child = child
-        self._start_point =  None
+        self._start_point = None
         self._end_point = None
         self._middle_points = []
         pen = QPen()
@@ -67,17 +66,46 @@ class Connection(QGraphicsPathItem, object):
         ''' Compute connection intermediate points - redefine in subclasses '''
         return self._middle_points
 
-    def arrow(self, path=None):
-        ''' Compute the two points of an arrow head - vertical by default '''
-        endp = self.end_point
+    def simple_arrow(self, origin='head', path=None):
+        ''' Compute the two points of an vertical arrow head '''
+        if origin == 'head':
+            endp = self.end_point
+        else:
+            endp = self.start_point
         return (QPointF(endp.x() - 5, endp.y() - 5),
                 QPointF(endp.x() + 5, endp.y() - 5))
 
-    def draw_arrow_head(self, shape):
-        ''' Generic function to draw any arrow head - don't redefine '''
-        arrowhead = self.arrow(shape)
+    def angle_arrow(self, path, origin='head'):
+        ''' Compute the two points of the arrow head with the right angle '''
+        if origin == 'tail':
+            path = path.toReversed()
+        length = path.length()
+        percent = path.percentAtLength(length - 10.0)
+        src = path.pointAtPercent(percent)
+        #path.moveTo(path.pointAtPercent(1))
+        end_point = path.pointAtPercent(1)
+        #end_point = path.currentPosition()
+        line = QLineF(src, end_point)
+        angle = math.acos(line.dx() / (line.length() or 1))
+        if line.dy() >= 0:
+            angle = math.pi * 2 - angle
+        arrow_size = 10.0
+        arrow_p1 = end_point + QPointF(
+                math.sin(angle - math.pi / 3) * arrow_size,
+                math.cos(angle - math.pi / 3) * arrow_size)
+        arrow_p2 = end_point + QPointF(
+                math.sin(angle - math.pi + math.pi / 3) * arrow_size,
+                math.cos(angle - math.pi + math.pi / 3) * arrow_size)
+        return (arrow_p1, arrow_p2)
+
+    def draw_arrow_head(self, shape, origin='head', kind='simple'):
+        ''' Generic function to draw a simple arrow '''
+        if kind == 'simple':
+            arrowhead = self.simple_arrow(path=shape)
+        else:
+            arrowhead = self.angle_arrow(shape, origin)
         shape.lineTo(arrowhead[0])
-        shape.moveTo(self.end_point)
+        shape.moveTo(self.end_point if origin == 'head' else self.start_point)
         shape.lineTo(arrowhead[1])
 
     def __str__(self):
@@ -94,7 +122,12 @@ class Connection(QGraphicsPathItem, object):
         shape.lineTo(self.end_point)
         # If required draw an arrow head (e.g. in SDL NEXTSTATE and JOIN)
         if self.child.arrow_head:
-            self.draw_arrow_head(shape)
+            self.draw_arrow_head(shape, origin='head',
+                                 kind=self.child.arrow_head)
+        if self.child.arrow_tail:
+            shape.moveTo(shape.pointAtPercent(0))
+            self.draw_arrow_head(shape, origin='tail',
+                                 kind=self.child.arrow_head)
         self.setPath(shape)
 
 
@@ -203,21 +236,63 @@ class CommentConnection(Connection):
 
 class Channel(Connection):
     ''' Subclass of Connection used to draw channels between processes '''
-    def __init__(self, elem1, elem2):
+    in_sig = out_sig = None
+    def __init__(self, process):
         ''' Set generic parameters from Connection class '''
-        super(Channel, self).__init__(elem1, elem2)
-        self.text_label = None
-        self.elem1 = elem1
-        self.elem2 = elem2
+        super(Channel, self).__init__(process, process)
+        self.label_in = QGraphicsTextItem('[]', parent=self)
+        self.label_out = QGraphicsTextItem('[]', parent=self)
+        if not Channel.in_sig:
+            # keep at class level as long as only one process is supported
+            # when copy-pasting a process the challel in/out signal lists
+            # are not parsed. Workaround is to keep the list "global"
+            # to allow a copy of both process and channel
+            # Needed for the image exporter, that copies the scene to a
+            # temporary one
+            Channel.in_sig = '[{}]'.format(',\n'.join(sig['name']
+                                   for sig in process.input_signals))
+            Channel.out_sig = '[{}]'.format(',\n'.join(sig['name']
+                                    for sig in process.output_signals))
+        font = QFont('Ubuntu', pointSize=8)
+        for each in (self.label_in, self.label_out):
+            each.setFont(font)
+            each.show()
+        self.process = process
+        self.reshape()
+
+    @property
+    def start_point(self):
+        ''' Compute connection origin - redefined function '''
+        parent_rect = self.process.boundingRect()
+        return QPointF(parent_rect.x(), parent_rect.height() / 2)
+
+    @property
+    def end_point(self):
+        ''' Compute connection end point - redefined function '''
+        # Arrow always bumps at the screen edge
+        try:
+            view = self.scene().views()[0]
+            view_pos = view.mapToScene(
+                           view.viewport().geometry()).boundingRect().topLeft()
+            scene_pos_x = self.mapFromScene(view_pos).x()
+            return QPointF(scene_pos_x, self.start_point.y())
+        except (IndexError, AttributeError):
+            # In case there is no view (e.g. Export PNG from cmd line)
+            return QPointF(self.start_point.x() - 250, self.start_point.y())
 
     def reshape(self):
-        ''' Update the shape of the connection line '''
+        ''' Redefine shape function to add the text areas '''
         super(Channel, self).reshape()
 
-    def paint(self, painter, option, widget):
-        ''' Apply antialiasing '''
-        painter.setRenderHint(QPainter.Antialiasing, True)
-        super(Channel, self).paint(painter, option, widget)
+        self.label_in.setPlainText(self.in_sig)
+        self.label_out.setPlainText(self.out_sig)
+
+        width_in = self.label_in.boundingRect().width()
+
+        self.label_in.setX(self.start_point.x() - width_in)
+        self.label_in.setY(self.start_point.y() + 5)
+        self.label_out.setX(self.end_point.x() + 10)
+        self.label_out.setY(self.end_point.y() + 5)
 
 
 class Controlpoint(QGraphicsPathItem, object):
@@ -261,7 +336,6 @@ class Connectionpoint(Controlpoint):
         # Symbol actually owning the connection point
         self.symbol = symbol
 
-
     def update_position(self):
         '''
             Update the position of the end point so that it is always
@@ -356,26 +430,7 @@ class Edge(Connection):
         ''' On a mouse click, display the control points '''
         self.bezier_set_visible(True)
 
-    def arrow(self, path):
-        ''' Compute the two points of the arrow head with the right angle '''
-        length = path.length()
-        percent = path.percentAtLength(length - 10.0)
-        src = path.pointAtPercent(percent)
-        end_point = path.currentPosition()
-        line = QLineF(src, end_point)
-        angle = math.acos(line.dx() / line.length())
-        if line.dy() >= 0:
-            angle = math.pi * 2 - angle
-        arrow_size = 10.0
-        arrow_p1 = end_point + QPointF(
-                math.sin(angle - math.pi/3) * arrow_size,
-                math.cos(angle - math.pi/3) * arrow_size)
-        arrow_p2 = end_point + QPointF(
-                math.sin(angle - math.pi + math.pi/3) * arrow_size,
-                math.cos(angle - math.pi + math.pi/3) * arrow_size)
-        return (arrow_p1, arrow_p2)
-
-   # pylint: disable=R0914
+  # pylint: disable=R0914
     def reshape(self):
         ''' Update the shape of the edge (redefined function) '''
         path = QPainterPath()
@@ -394,7 +449,7 @@ class Edge(Connection):
             path.lineTo(self.end_connection.center)
 
         end_point = path.currentPosition()
-        arrowhead = self.arrow(path)
+        arrowhead = self.angle_arrow(path)
         path.lineTo(arrowhead[0])
         path.moveTo(end_point)
         path.lineTo(arrowhead[1])
