@@ -1,12 +1,11 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
- 
+
 """
     This module is a free SDL editor.
     It allows to graphically design state machines, using a formal, well-
     defined language, and generate Ada code from the models.
 """
- 
-__version__ = "0.992"
 
-from opengeode import opengeode
+
+from opengeode import opengeode, __version__
