@@ -41,6 +41,7 @@ LOG = logging.getLogger(__name__)
 class Expression(object):
     ''' AST Entry for expressions - Always use subtype '''
     is_raw = False
+
     def __init__(self, inputString='', line=-1, charPositionInLine=-1):
         ''' Initialize Expression attributes '''
         self.inputString = inputString
@@ -54,9 +55,9 @@ class Expression(object):
         # Hint for code generators: intermediate storage identifier
         self.tmpVar = -1
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for an expression '''
-        return '{exp} ({l},{c})'.format(exp=self.inputString,
+        return u'{exp} ({l},{c})'.format(exp=self.inputString,
                                         l=self.line,
                                         c=self.charPositionInLine)
 
@@ -75,14 +76,19 @@ class ExprMinus(Expression):
 
 class ExprOr(Expression):
     operand = 'or'
+    # Indicate that short circuit form is used ("or else")
+    shortcircuit = ''
 
 
 class ExprAnd(Expression):
     operand = 'and'
+    # Indicate that short circuit form is used ("and then")
+    shortcircuit = ''
 
 
 class ExprXor(Expression):
     operand = 'xor'
+    shortcircuit = ''
 
 
 class ExprEq(Expression):
@@ -118,16 +124,32 @@ class ExprMod(Expression):
 
 
 class ExprRem(Expression):
-    operand = 'mod'
+    operand = 'rem'
+
+
+class ExprNot(Expression):
+    operand = 'not'
+
+
+class ExprNeg(Expression):
+    operand = '-'
 
 
 class ExprAssign(Expression):
     operand = ':='
 
 
-class ExprAppend(Expression): pass
-class ExprIn(Expression): pass
-class ExprImplies(Expression): pass
+class ExprAppend(Expression):
+    pass
+
+
+class ExprIn(Expression):
+    pass
+
+
+class ExprImplies(Expression):
+    pass
+
 
 class Primary(Expression):
     '''
@@ -135,6 +157,7 @@ class Primary(Expression):
         Abstract class, never used directly, see subtypes below
     '''
     is_raw = True
+
     def __init__(self, inputString='', line=-1, charPositionInLine=-1,
                  primary=None):
         ''' Initialize common primary attributes '''
@@ -142,24 +165,21 @@ class Primary(Expression):
             self.inputString = primary.inputString
             self.line = primary.line
             self.charPositionInLine = primary.charPositionInLine
-            self.op_not = primary.op_not
-            self.op_minus = primary.op_minus
             self.value = primary.value
             self.exprType = primary.exprType
         else:
             self.inputString = inputString
             self.line = line
             self.charPositionInLine = charPositionInLine
-            self.op_not = False
-            self.op_minus = False
             self.value = None
             self.exprType = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a primary '''
-        return 'PRIMARY : {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'PRIMARY : {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
+
 # Subclasses of Primary - never use Primary directly
 class PrimPath(Primary):
     ''' PrimPath is a list of elements needed to identify a value
@@ -172,15 +192,43 @@ class PrimPath(Primary):
     '''
     is_raw = False
 
-class PrimVariable(PrimPath): pass # XXX should not be raw for codegen
-class PrimFPAR(PrimVariable): pass
-class PrimEnumeratedValue(Primary): pass
-class PrimInteger(Primary): pass
-class PrimReal(Primary): pass
-class PrimBoolean(Primary): pass
-class PrimConstant(Primary): is_raw = False #pass
-class PrimBitStringLiteral(Primary): pass   # Not supported yet
-class PrimOctetStringLiteral(Primary): pass # Not supported yet
+
+class PrimVariable(PrimPath):
+    pass
+
+
+class PrimFPAR(PrimVariable):
+    pass
+
+
+class PrimEnumeratedValue(Primary):
+    pass
+
+
+class PrimInteger(Primary):
+    pass
+
+
+class PrimReal(Primary):
+    pass
+
+
+class PrimBoolean(Primary):
+    pass
+
+
+class PrimConstant(Primary):
+    is_raw = False
+
+
+class PrimBitStringLiteral(Primary):
+    ''' Not supported yet '''
+    pass
+
+
+class PrimOctetStringLiteral(Primary):
+    ''' Not supported yet '''
+    pass
 
 
 class PrimIfThenElse(Primary):
@@ -213,7 +261,8 @@ class PrimMantissaBaseExp(Primary):
     pass
 
 
-class PrimEmptyString(Primary): pass
+class PrimEmptyString(Primary):
+    pass
 
 
 class PrimChoiceItem(Primary):
@@ -221,7 +270,8 @@ class PrimChoiceItem(Primary):
     pass
 
 
-class PrimChoiceDeterminant(Primary): pass
+class PrimChoiceDeterminant(Primary):
+    pass
 
 
 class PrimSequenceOf(Primary):
@@ -259,9 +309,9 @@ class Decision(object):
         # hint for backends needing a temporary variable to hold the question
         self.tmpVar = -1
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a decision '''
-        return 'DECISION {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'DECISION {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -294,9 +344,9 @@ class Answer(object):
         # optional hyperlink
         self.hyperlink = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for an answer '''
-        return 'ANSWER {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'ANSWER {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -317,9 +367,9 @@ class Task(object):
         self.hyperlink = None
         self.elems = []
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a task '''
-        return 'TASK {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'TASK {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -341,6 +391,7 @@ class TaskForLoop(Task):
     '''
     pass
 
+
 class Output(object):
     ''' AST Entry for OUTPUT statements '''
     def __init__(self, defName=''):
@@ -359,9 +410,9 @@ class Output(object):
         # optional hyperlink
         self.hyperlink = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for an Output symbol '''
-        return '{exp} ({l},{c})'.format(exp=self.inputString,
+        return u'{exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -398,10 +449,18 @@ class Terminator(object):
         self.entrypoint = None
         # some transitions can be chained, when entering/leaving nested states
         self.next_id = -1
+        # Pointer to the next transition, when using return/connect
+        self.next_trans = None
+        # List of State that can lead to this terminator
+        # There can be several if terminator follows a floating label
+        # Note, this field is updated by the Helper.flatten function
+        self.possible_states = []
+        # optional composite state content (type CompositeState)
+        self.composite = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for terminators '''
-        return '{kind} {exp} ({l},{c}) at {x}, {y}'.format(
+        return u'{kind} {exp} ({l},{c}) at {x}, {y}'.format(
                 exp=self.inputString,
                 kind=self.kind.upper(), l=self.line, c=self.charPositionInLine,
                 x=self.pos_x, y=self.pos_y)
@@ -428,9 +487,9 @@ class Label(object):
         # Transition is used for floating labels (see AST entry)
         self.transition = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a label '''
-        return 'LABEL {label} ({l},{c})'.format(label=self.inputString,
+        return u'LABEL {label} ({l},{c})'.format(label=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -450,9 +509,9 @@ class Floating_label(Label, object):
             self.terminators = label.terminators
             self.transition = label.transition
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a label (used by code generator) '''
-        return 'CONNECTION {label} ({l},{c})'.format(label=self.inputString,
+        return u'CONNECTION {label} ({l},{c})'.format(label=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -466,14 +525,19 @@ class Transition(object):
         self.terminator = None
         # All Terminators of this transition
         self.terminators = []
+        # List of State that can lead to this transition
+        # There can be several if state has multiple names (e.g. STATE a, b)
+        # Note, this field is updated by the Helper.flatten function
+        # and is needed to properly know when to call a nested state exit
+        # procedure when the model is flattened.
+        self.possible_states = []
 
-
-    def __repr__(self):
+    def trace(self):
         ''' Debug output: display all actions '''
-        data = [repr(action) for action in self.actions]
+        data = [action.trace() for action in self.actions]
         if self.terminator:
-            data.append(repr(self.terminator))
-        return '\n'.join(data)
+            data.append(self.terminator.trace())
+        return u'\n'.join(data)
 
 
 class Input(object):
@@ -506,9 +570,9 @@ class Input(object):
         # list of terminators following the input symbol
         self.terminators = []
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for an INPUT symbol '''
-        return 'INPUT {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'INPUT {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -521,9 +585,9 @@ class Connect(Input):
         self.connect_list = []
         self.width = 1
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a CONNECT symbol '''
-        return 'CONNECT {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'CONNECT {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -545,9 +609,9 @@ class Start(object):
         # list of terminators following the start symbol
         self.terminators = []
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a START symbol '''
-        return 'START {}'.format(self.inputString)
+        return u'START {}'.format(self.inputString)
 
 
 class Procedure_start(Start):
@@ -575,9 +639,10 @@ class Comment(object):
         # optional hyperlink
         self.hyperlink = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a COMMENT symbol '''
-        return 'COMMENT {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'COMMENT {exp} ({l},{c})'.format(
+                exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -606,9 +671,9 @@ class State(object):
         # optional composite state content (type CompositeState)
         self.composite = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a STATE symbol '''
-        return 'STATE {exp} ({l},{c}) at {x},{y}'.format(exp=self.inputString,
+        return u'STATE {exp} ({l},{c}) at {x},{y}'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine,
                 x=self.pos_x, y=self.pos_y)
 
@@ -631,9 +696,9 @@ class TextArea(object):
         # optional hyperlink
         self.hyperlink = None
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for a text area '''
-        return 'TEXTAREA {exp} ({l},{c})'.format(exp=self.inputString,
+        return u'TEXTAREA {exp} ({l},{c})'.format(exp=self.inputString,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -665,8 +730,10 @@ class Procedure(object):
         self.hyperlink = None
         # Local variables dictionnary (see Process)
         self.variables = {}
-        # Inherited variables from all levels above
+        self.timers = []
+        # Inherited variables and timers from all levels above
         self.global_variables = {}
+        self.global_timers = []
         # Formal parameters - list of dict:
         # [{'name': str, 'direction': 'in'/'out', 'type': str}]
         self.fpar = []
@@ -695,6 +762,9 @@ class Procedure(object):
         self.comment = None
         # Set of symbols contained in the procedure (type Automaton)
         self.content = Automaton(parent=self)
+        # input/output signal lists - unused but for context information
+        self.input_signals = []
+        self.output_signals = []
 
 
 class Process(object):
@@ -710,11 +780,12 @@ class Process(object):
         self.referenced = False
         # variables: dictionnary: {variable1Name: (asn1SccType, default value)}
         self.variables = {}
-        # global variables can be used to inherit variables
+        # global variables and timers can be used to inherit from a level above
         self.global_variables = {}
-        
+        self.global_timers = []
         # Set default coordinates and width/height
-        self.pos_x = self.pos_y = 150
+        self.pos_x = 250
+        self.pos_y = 150
         self.width = 150
         self.height = 75
         # Optional hyperlink
@@ -740,7 +811,7 @@ class Process(object):
 
         # list of operators (not supported) and procedures
         self.operators = {}
-        # list of Procedure (external procedures)
+        # list of Procedure (external and inner procedures)
         self.procedures = []
 
         # The Mapping structure should be used for code generation backends
@@ -781,9 +852,9 @@ class CompositeState(Process):
         # one nameless START, named START (one per entrypoint), states,
         # amd floating labels
 
-    def __repr__(self):
+    def trace(self):
         ''' Debug output for composite state '''
-        return 'COMPOSITE STATE {exp} ({l},{c})'.format(exp=self.statename,
+        return u'COMPOSITE STATE {exp} ({l},{c})'.format(exp=self.statename,
                 l=self.line, c=self.charPositionInLine)
 
 
@@ -830,7 +901,7 @@ class System(object):
         self.signals = []
         # list of ogAST.Procedure
         self.procedures = []
-        # channels: [{'name':str, 'routes':[{'source': str, 'dest': str, 
+        # channels: [{'name':str, 'routes':[{'source': str, 'dest': str,
         # 'signals': ['sig1', .. ]}]
         self.channels = []
         self.blocks = []
