@@ -20,7 +20,7 @@
 """
 
 import logging
-import math
+import re
 from PySide import QtGui, QtCore
 
 try:
@@ -31,6 +31,7 @@ except ImportError:
           'the official repos: sudo apt-get install python-pygraphviz')
 
 import genericSymbols
+from Connectors import Edge
 
 RENDER_DPI = {'X': 93.0, 'Y': 95.0}
 G_SYMBOLS = set()
@@ -60,6 +61,8 @@ class Record(genericSymbols.HorizontalSymbol, object):
             property_box = QtGui.QGraphicsTextItem(self)
             property_box.setPos(5, 30)
             property_box.setPlainText(node['properties'])
+        # Text in statecharts is read-only:
+        self.text.setTextInteractionFlags(QtCore.Qt.TextBrowserInteraction)
 
     def set_shape(self, width, height):
         ''' Define the polygon shape from width and height '''
@@ -93,15 +96,17 @@ class Point(genericSymbols.HorizontalSymbol, object):
     _terminal_followers = []
     textbox_alignment = (QtCore.Qt.AlignTop
                          | QtCore.Qt.AlignHCenter)
+    has_text_area = False
 
     def __init__(self, node, graph):
         ''' Initialization: compute the polygon shape '''
         self.name = node['name']
-        super(Point, self).__init__(x=node['pos'][0],
-                y=node['pos'][1], text='')
+        super(Point, self).__init__(x=node['pos'][0], y=node['pos'][1])
         self.set_shape(node['width'], node['height'])
         self.setBrush(QtGui.QBrush(QtCore.Qt.black))
         self.graph = graph
+        # Text is read only
+        #self.text.setTextInteractionFlags(QtCore.Qt.TextBrowserInteraction)
 
     def set_shape(self, width, height):
         ''' Define the polygon shape from width and height '''
@@ -133,12 +138,12 @@ class Diamond(genericSymbols.HorizontalSymbol, object):
     _terminal_followers = []
     textbox_alignment = (QtCore.Qt.AlignTop
                          | QtCore.Qt.AlignHCenter)
+    has_text_area = False
 
     def __init__(self, node, graph):
         ''' Initialization: compute the polygon shape '''
         self.name = node['name']
-        super(Diamond, self).__init__(x=node['pos'][0],
-                y=node['pos'][1], text='')
+        super(Diamond, self).__init__(x=node['pos'][0], y=node['pos'][1])
         self.set_shape(node['width'], node['height'])
         self.graph = graph
 
@@ -168,214 +173,6 @@ class Diamond(genericSymbols.HorizontalSymbol, object):
         pass
 
 
-class Controlpoint(QtGui.QGraphicsPathItem, object):
-    ''' Class handling one edge control point (to change bezier curves) '''
-    def __init__(self, pos, edge):
-        ''' Set the original control point - with color, shape '''
-        path = QtGui.QPainterPath()
-        path.addEllipse(pos.x() - 5, pos.y() - 5, 10, 10)
-        super(Controlpoint, self).__init__(path, parent=edge)
-        self.setPen(QtGui.QColor(50, 100, 120, 200))
-        self.setBrush(QtGui.QColor(200, 200, 210, 120))
-        self.setFlags(QtGui.QGraphicsItem.ItemIsSelectable |
-                      QtGui.QGraphicsItem.ItemIsMovable)
-        self.edge = edge
-        self.hide()
-
-    @property
-    def center(self):
-        ''' Return the position of the center of the shape '''
-        return self.pos() + self.boundingRect().center()
-
-    def mouseMoveEvent(self, event):
-        ''' When user moves a control point, update the connection shape '''
-        super(Controlpoint, self).mouseMoveEvent(event)
-        self.edge.reshape()
-
-
-class Connectionpoint(Controlpoint, object):
-    '''
-        Class handling an end connection point - similar to Controlpoint,
-        except the shape and the behaviour when moved. Connection point
-        must check with its owning symbol if the move is valid.
-    '''
-    def __init__(self, pos, edge, symbol):
-        ''' Create the point - as a small, lightblue box '''
-        super(Connectionpoint, self).__init__(pos, edge=edge)
-        path = QtGui.QPainterPath()
-        path.addRect(0, 0, 10, 10)
-        self.setPath(path)
-        self.setPos(pos.x() - 5, pos.y() - 5)
-        # Symbol actually owning the connection point
-        self.symbol = symbol
-
-
-    def update_position(self):
-        '''
-            Update the position of the end point so that it is always
-            placed correctly on the symbol that contains it (but that may
-            not be the parent of the point if is the end point)
-            After the position update, reshape the edge so that the lines
-            are properly drawn to the new position
-        '''
-        # Transform position to owning symbol coordinates:
-        center_pos = self.scenePos() + self.boundingRect().center()
-        pos_symb = self.symbol.mapFromScene(center_pos)
-
-        # Ask to symbol the nearest valid position on its shape
-        # to place the connection point, and adjust position
-        nearest_x, nearest_y = self.symbol.closest_connection_point(pos_symb)
-
-        self.moveBy(-nearest_x, -nearest_y)
-
-        self.edge.reshape()
-
-    def mouseMoveEvent(self, event):
-        '''
-            When user moves the point, check with the connected symbol
-            where the actual connection point should be placed
-        '''
-        # calling super function moves to the new point
-        super(Connectionpoint, self).mouseMoveEvent(event)
-        # Then compute a valid position, based on the owning symbol
-        self.update_position()
-
-
-class Edge(genericSymbols.Connection, object):
-    ''' B-spline/Bezier connection shape '''
-    def __init__(self, edge, graph):
-        ''' Set generic parameters from Connection class '''
-        self.text_label = None
-        super(Edge, self).__init__(edge['source'], edge['target'])
-        self.edge = edge
-        self.graph = graph
-
-        # Initialize control point coordinates
-        # Start and End points are optional - graphviz decision
-        self.start_point = (self.mapFromScene(*self.edge['start']) if
-                 self.edge.get('start') else None)
-        self.end_point = (self.mapFromScene(*self.edge['end']) if
-                 self.edge.get('end') else None)
-        self.bezier = [self.mapFromScene(*self.edge['spline'][0])]
-        # Bezier control points (groups of three points):
-        assert(len(self.edge['spline']) % 3 == 1)
-        for i in xrange(1, len(self.edge['spline']), 3):
-            self.bezier.append([Controlpoint(
-                          self.mapFromScene(*self.edge['spline'][i + j]), self)
-                          for j in range(3)])
-        # Set connection points as not visible, by default
-        self.bezier_visible = False
-        
-        # Create connection points at start and end of the edge
-        self.source_connection = Connectionpoint(
-                self.start_point or self.bezier[0], self, self.parent)
-        self.parent.movable_points.append(self.source_connection)
-        self.end_connection = Connectionpoint(
-                self.end_point or self.bezier[-1], self, self.child)
-        self.child.movable_points.append(self.end_connection)
-        self.reshape()
-
-    def bezier_set_visible(self, visible=True):
-        ''' Display or hide the edge control points '''
-        self.bezier_visible = visible
-        for group in self.bezier[1:]:
-            for ctrl_point in group:
-                if visible:
-                    ctrl_point.show()
-                else:
-                    ctrl_point.hide()
-        if visible:
-            self.end_connection.show()
-            self.source_connection.show()
-        else:
-            self.end_connection.hide()
-            self.source_connection.hide()
-        self.update()
-
-    def mousePressEvent(self, event):
-        ''' On a mouse click, display the control points '''
-        self.bezier_set_visible(True)
-
-   # pylint: disable=R0914
-    def reshape(self):
-        ''' Update the shape of the edge (redefined function) '''
-        path = QtGui.QPainterPath()
-        # If there is a starting point, draw a line to the first curve point
-        if self.start_point:
-            path.moveTo(self.source_connection.center) #start_point)
-            path.lineTo(self.bezier[0])
-        else:
-            path.moveTo(self.source_connection.center)#bezier[0])
-        # Loop over the curve points:
-        for group in self.bezier[1:]:
-            path.cubicTo(*[point.center for point in group])
-
-        # If there is an ending point, draw a line to it
-        if self.end_point:
-            path.lineTo(self.end_connection.center) #end_point)
-
-        # Draw the arrow head
-        length = path.length()
-        percent = path.percentAtLength(length - 10.0)
-        src = path.pointAtPercent(percent)
-        #angle = path.angleAtPercent(percent)
-        #print angle
-        end_point = path.currentPosition()
-        line = QtCore.QLineF(src, end_point)
-        angle = math.acos(line.dx() / line.length())
-        if line.dy() >= 0:
-            angle = math.pi * 2 - angle
-        arrow_size = 10.0
-        arrow_p1 = end_point + QtCore.QPointF(
-                math.sin(angle - math.pi/3) * arrow_size,
-                math.cos(angle - math.pi/3) * arrow_size)
-        arrow_p2 = end_point + QtCore.QPointF(
-                math.sin(angle - math.pi + math.pi/3) * arrow_size,
-                math.cos(angle - math.pi + math.pi/3) * arrow_size)
-        path.lineTo(arrow_p1)
-        path.lineTo(end_point)
-        path.lineTo(arrow_p2)
-        path.moveTo(end_point)
-        try:
-            # Add the transition label, if any (none for the START edge)
-            font = QtGui.QFont('arial', pointSize=8)
-            width = QtGui.QFontMetrics(font).width(
-                    self.edge.get('label', 0))
-            pos = self.mapFromScene(*self.edge['lp'])
-            #path.addText(pos.x() - width/2, pos.y(),
-            #        font, self.edge['label'])
-            if not self.text_label:
-                self.text_label = QtGui.QGraphicsTextItem(
-                                 self.edge.get('label', ''), parent=self)
-            self.text_label.setX(pos.x() - width / 2)
-            self.text_label.setY(pos.y())
-            self.text_label.setFont(font)
-            self.text_label.show()
-        except KeyError:
-            # no label
-            pass
-        self.setPath(path)
-
-
-    def __str__(self):
-        ''' user-friendly information about the edge coordinates '''
-        return('Edge between ' + self.edge['source'].name + ' and ' +
-                self.edge['target'].name + str(self.edge['spline'][0]))
-
-    def paint(self, painter, option, widget):
-        ''' Apply anti-aliasing to Edge Connections '''
-        painter.setRenderHint(QtGui.QPainter.Antialiasing, True)
-        super(Edge, self).paint(painter, option, widget)
-        # Draw lines between connection points, if visible
-        if self.bezier_visible:
-            painter.setPen(
-                    QtGui.QPen(QtCore.Qt.lightGray, 0, QtCore.Qt.SolidLine))
-            painter.setBrush(QtCore.Qt.NoBrush)
-            points_flat = [point.center
-                           for sub1 in self.bezier[1:] for point in sub1]
-            painter.drawPolyline([self.source_connection.center]
-                                  + points_flat + [self.end_connection.center])
-
 def edges(scene, node):
     ''' Return all edges of a given node '''
     for item in scene.items():
@@ -445,31 +242,54 @@ def preprocess_nodes(my_graph, bounding_rect, dpi):
     # while Qt uses the top-left corner, so we must translate all y-coord.
     bb_height = bounding_rect[3]
     nodes = []
+    attrs = []
+    for each in my_graph.subgraphs():
+        # Transform cluster nodes into nodes (attributes differ)
+        llx, lly, urx, ury = each.graph_attr['bb'].split(',')
+        wid = float(urx) - float(llx)
+        hei = float(ury) - float(lly)
+        attr = {}
+        attr.update(pos='{0},{1}'.format(llx, lly),
+                         width=wid,
+                         height=hei,
+                         shape=each.graph_attr['shape'],
+                         label=each.graph_attr['label'],
+                         style=each.graph_attr['style'],
+                         name='cluster_' + each.graph_attr['label'],
+                         kind='cluster')
+        attrs.append(attr)
     for node in my_graph.nodes_iter():
+        if not node.attr:
+            continue
+        node.attr.update(name=node.name)
+        attrs.append(node.attr)
+    for node in attrs:
         new_node = {}
         # Get the node shape attribute - default is record
-        new_node['shape'] = node.attr.get('shape') or 'record'
+        new_node['shape'] = node.get('shape') or 'record'
         # node main name
-        new_node['name'] = str(node)
+        new_node['name'] = node['name']
         try:
             # node complete label (can contain several compartments)
             properties = (
-                    node.attr['label'][1:-2].split('|')[1].replace('\\', '\n'))
-            new_node['properties'] = properties  # node.attr.get('label')
+                    node['label'][1:-2].split('|')[1].replace('\\', '\n'))
+            new_node['properties'] = properties
         except IndexError:
             pass
         # transform width and height from inches to pixels
-        new_node['width'] = float(node.attr.get('width')) * RENDER_DPI['X']
-        new_node['height'] = float(node.attr.get('height')) * RENDER_DPI['Y']
+        if node.get('kind', '') == 'cluster':
+            new_node['width'] = node['width']
+            new_node['height'] = node['height']
+        else:
+            new_node['width'] = float(node['width']) * RENDER_DPI['X']
+            new_node['height'] = float(node['height']) * RENDER_DPI['Y']
         # get the position of the CENTER of the node
-        center_pos = [float(val)
-                for val in node.attr.get('pos').split(',')]
+        center_pos = [float(val) for val in node['pos'].split(',')]
         # apply dpi-conversion from 72 to 96
         center_pos[0] *= (RENDER_DPI['X'] / dpi)
 
         # translate y-coord from bottom-left to top-left corner
-        center_pos[1] = (
-                bb_height - center_pos[1]) * (RENDER_DPI['Y'] / dpi)
+        center_pos[1] = (bb_height - center_pos[1]) * (RENDER_DPI['Y'] / dpi)
         new_node['pos'] = [center_pos[0] - (new_node['width'] / 2.0),
                            center_pos[1] - (new_node['height'] / 2.0)]
         nodes.append(new_node)
@@ -501,16 +321,16 @@ def update(scene):
         center_x = center_pos.x() * (dpi / RENDER_DPI['X'])
         center_y = (bb_height - center_pos.y()) * (dpi / RENDER_DPI['Y'])
 
-        pos = unicode('{x},{y}'.format(x=int(center_x), y=int(center_y)))
-
-        #print node['name'], node['height'], node['height'] / RENDER_DPI['Y']
+        #pos = unicode('{x},{y}'.format(x=int(center_x), y=int(center_y)))
+        pos = unicode('{x},{y}'.format(x=float(center_x), y=float(center_y)))
 
         if node['shape'] in (Point, Diamond):
             graph.add_node(node['name'], pos=pos, shape=lookup[node['shape']],
                 fixedsize='true', width=node['width'] / RENDER_DPI['X'],
-                height=node['height'] / RENDER_DPI['Y'])
+                height=node['height'] / RENDER_DPI['Y'], pin=True)
         else:
-            graph.add_node(node['name'], pos=pos, shape=lookup[node['shape']])
+            graph.add_node(node['name'], pos=pos, pin=True,
+                           shape=lookup[node['shape']])
 
     # Keep edges from the previous graph
     for edge in EDGES:
@@ -518,14 +338,16 @@ def update(scene):
 
     #print graph.to_string()
     if nodes:
-        #before = scene.itemsBoundingRect().center()
+        before = scene.itemsBoundingRect().center()
+        #before_pos = graph.get_node(nodes[0]['name']).attr['pos']
         render_statechart(scene, graph, keep_pos=True)
-        #delta = scene.itemsBoundingRect().center() - before
+        #after_pos = graph.get_node(nodes[0]['name']).attr['pos']
+        #print before_pos,after_pos
+        delta = scene.itemsBoundingRect().center() - before
         # graphviz translates the graph to pos (0, 0) -> move it back
         # to the exact graphical position where the user clicked
-        #for item in scene.items():
-        #    if isinstance(item, genericSymbols.Symbol):
-        #        item.setPos(item.pos() - delta)
+        for item in scene.visible_symb:
+            item.setPos(item.pos() - delta)
 
 
 def render_statechart(scene, graph=None, keep_pos=False):
@@ -541,8 +363,10 @@ def render_statechart(scene, graph=None, keep_pos=False):
     graph.graph_attr.update(dpi='72.0')
     EDGES[:] = graph.edges()
 
-    scene.clear()
-    G_SYMBOLS.clear()
+    for each in scene.visible_symb:
+        each.setVisible(False)
+    #scene.clear()
+    #G_SYMBOLS.clear()
 
     # Compute all the coordinates (self-modifying function)
     # Force the fontsize of the nodes to be 12, as in OpenGEODE
@@ -568,19 +392,110 @@ def render_statechart(scene, graph=None, keep_pos=False):
     nodes = preprocess_nodes(graph, bounding_rect, dot_dpi)
     node_symbols = []
     for node in nodes:
+        #print node
         shape = node.get('shape')
         try:
             node_symbol = lookup[shape](node, graph)
+            G_SYMBOLS.add(node_symbol)
+            node_symbols.append(node_symbol)
+            scene.addItem(node_symbol)
         except KeyError:
             raise TypeError('Statechart - unsupported shape: ' + shape)
-        G_SYMBOLS.add(node_symbol)
-        node_symbols.append(node_symbol)
-        scene.addItem(node_symbol)
     edges = preprocess_edges(graph, node_symbols, bounding_rect, dot_dpi)
 
     for edge in edges:
         Edge(edge, graph)
 
 
+def create_dot_graph(root_ast):
+    ''' Return a dot.AGraph item, from an ogAST.Process or child entry '''
+    graph = dotgraph.AGraph(strict=False, directed=True)
+    diamond = 0
+    for state in root_ast.mapping.viewkeys():
+        # create a new node for each state (including nested states)
+        if state.endswith('START'):
+            graph.add_node(state, label='', shape='point',
+                           fixedsize='true', width=10.0 / 72.0)
+        else:
+            #print 'adding', state
+            graph.add_node(state, label=state, shape='record', style='rounded')
+    for each in root_ast.composite_states:
+        # this will have to be recursive
+        subnodes = (name for name in graph.iternodes()
+                    if name.startswith(each.statename.lower() + '_'))
+        graph.add_subgraph(subnodes, name='cluster_' + each.statename.lower(),
+                           label=each.statename.lower(),
+                           style='rounded', shape='record')
+    for state, inputs in root_ast.mapping.viewitems():
+        # Add edges
+        transitions = \
+            inputs if not state.endswith('START') \
+                   else [root_ast.transitions[inputs]]
+                   #[root_ast.content.start]
+        for trans in transitions:
+            source = state
+            # transition label - there can be several inputs
+            try:
+                # Keep only message name, remove params and newlines
+                # (newlines are not supported by graphviz)
+                label, = re.match(r'([^(]+)', trans.inputString).groups()
+                label = label.strip().replace('\n', ' ')
+            except AttributeError:
+                # START transition may have no inputString
+                label = ''
+
+            def find_terminators(trans):
+                ''' Recursively find all NEXTSTATES '''
+                next_states = [term for term in trans.terminators
+                               if term.kind == 'next_state']
+                joins = [term for term in trans.terminators
+                         if term.kind == 'join']
+                for join in joins:
+                    # JOIN - Find corresponding label
+                    try:
+                        corr_label, = [lab for lab in
+                                       root_ast.content.floating_labels +
+                                       root_ast.labels if
+                                       lab.inputString.lower() ==
+                                       join.inputString.lower()]
+                    except ValueError:
+                        LOG.error('Missing label: ' + join.inputString)
+                    else:
+                        # Don't recurse forever in case of livelock
+                        if corr_label.inputString != trans.inputString:
+                            next_states.extend(find_terminators(corr_label))
+                return set(next_states)
+
+            # Determine the list of terminators in this transition
+            next_states = find_terminators(trans)
+
+            if len(next_states) > 1:
+                # more than one terminator - add intermediate node
+                graph.add_node(str(diamond),
+                               shape='diamond',
+                               fixedsize='true',
+                               width=15.0 / 72.0,
+                               height=15.0 / 72.0, label='')
+                graph.add_edge(source, str(diamond), label=label)
+                source = str(diamond)
+                label = ''
+                diamond += 1
+            for term in next_states:
+                if term.inputString.strip() == '-':
+                    target = state
+                else:
+                    target = term.inputString.lower()
+                LOG.debug('Edge from ' + source + ' to ' +
+                           term.inputString + ' label: ' + label)
+                for each in root_ast.composite_states:
+                    # check with deeper nesting
+                    if each.statename.lower() == target.lower():
+                        target = 'cluster_' + target
+                        break
+                graph.add_edge(source, target, label=label)
+    #print graph.to_string()
+    return graph
+
+
 if __name__ == '__main__':
     render_statechart(None)
