@@ -22,14 +22,11 @@ __all__ = ['Input', 'Output', 'State', 'Task', 'ProcedureCall', 'Label',
            'Procedure', 'ProcedureStart', 'ProcedureStop', 'ASN1Viewer',
            'StateStart', 'Process']
 
-import traceback
+#import traceback
 import logging
-from itertools import chain
-from collections import deque
 
 from PySide.QtCore import Qt, QPoint, QRect, QRectF
-from PySide.QtGui import(QPainterPath, QBrush, QColor, QPolygon,
-        QRadialGradient, QPainter, QPolygonF, QPen)
+from PySide.QtGui import(QPainterPath, QBrush, QColor, QRadialGradient, QPen)
 
 from genericSymbols import HorizontalSymbol, VerticalSymbol, Comment
 from Connectors import Connection, JoinConnection, Channel
@@ -48,7 +45,9 @@ SDL_BLACKBOLD = ['\\b{word}\\b'.format(word=word) for word in (
                 'EXPONENT', 'TRUE', 'FALSE', 'MOD', 'FI', 'WRITE', 'WRITELN',
                 'LENGTH', 'PRESENT', 'FPAR', 'TODO', 'FIXME', 'XXX',
                 'CHECKME', 'PROCEDURE', 'EXTERNAL', 'IN', 'OUT', 'TIMER',
-                'SET_TIMER', 'RESET_TIMER', 'VIA', 'ENTRY', 'EXIT')]
+                'SET_TIMER', 'RESET_TIMER', 'VIA', 'ENTRY', 'EXIT',
+                'SYNTYPE', 'ENDSYNTYPE', 'CONSTANTS',
+                'NEWTYPE', 'ENDNEWTYPE', 'ARRAY', 'STRUCT', 'SYNONYM')]
 
 SDL_REDBOLD = ['\\b{word}\\b'.format(word=word) for word in (
               'INPUT', 'OUTPUT', 'STATE', 'DECISION', 'NEXTSTATE',
@@ -60,7 +59,7 @@ class Input(HorizontalSymbol):
     ''' SDL INPUT Symbol '''
     _unique_followers = ['Comment']
     _insertable_followers = ['Task', 'ProcedureCall', 'Output', 'Decision',
-                        'Input',  'Label', 'Connect']
+                             'Input', 'Label', 'Connect']
     _terminal_followers = ['Join', 'State', 'ProcedureStop']
     completion_list = set()
 
@@ -88,15 +87,6 @@ class Input(HorizontalSymbol):
         if ast.comment:
             Comment(parent=self, ast=ast.comment)
 
-    def check_syntax(self):
-        ''' Redefined function, to check only the symbol, not recursively '''
-        _, syntax_errors, ___, ____, _____ = self.parser.parseSingleElement(
-                self.common_name, self.pr())
-        try:
-            self.scene().raise_syntax_errors(syntax_errors)
-        except AttributeError:
-            LOG.debug('raise_syntax_error raised an exception')
-
     def insert_symbol(self, parent, x, y):
         ''' Insert Input symbol - propagate branch Entry point '''
         # Make sure that parent is a state, not a sibling input
@@ -116,28 +106,6 @@ class Input(HorizontalSymbol):
         self.setPath(path)
         super(Input, self).set_shape(width, height)
 
-    def pr(self):
-        ''' Return the PR notation of the single INPUT symbol '''
-        comment = repr(self.comment) if self.comment else ';'
-        pos = self.scenePos()
-        return('/* CIF INPUT ({x}, {y}), ({w}, {h}) */\n'
-                '{hlink}'
-                'INPUT {i}{comment}'.format(
-                    hlink=repr(self.text), i=str(self),
-                    x=int(pos.x()), y=int(pos.y()),
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height()), comment=comment))
-
-    def __repr__(self):
-        ''' Return the complete Input branch in PR format '''
-        result = [self.pr()]
-        # Recursively return the complete branch below the INPUT
-        next_symbol = self.next_aligned_symbol()
-        while next_symbol:
-            result.append(repr(next_symbol))
-            next_symbol = next_symbol.next_aligned_symbol()
-        return '\n'.join(result)
-
 
 class Connect(Input):
     ''' Connect point below a nested state '''
@@ -146,6 +114,7 @@ class Connect(Input):
     resizeable = False
     # Symbol must not use antialiasing, otherwise the middle line is too thick
     _antialiasing = False
+
     def set_shape(self, width, height):
         ''' Compute the polygon to fit in width, height '''
         self.setPen(QPen(Qt.blue))
@@ -162,19 +131,6 @@ class Connect(Input):
         ''' Symbol cannot be resized '''
         return
 
-    def pr(self):
-        ''' Return the PR notation of the single CONNECT symbol '''
-        comment = repr(self.comment) if self.comment else ';'
-        pos = self.scenePos()
-        pr_string =('/* CIF CONNECT ({x}, {y}), ({w}, {h}) */\n'
-                '{hlink}'
-                'CONNECT {i}{comment}'.format(
-                    hlink=repr(self.text), i=str(self.text),
-                    x=int(pos.x()), y=int(pos.y()),
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height()), comment=comment))
-        return pr_string
-
 
 # pylint: disable=R0904
 class Output(VerticalSymbol):
@@ -213,25 +169,13 @@ class Output(VerticalSymbol):
         self.setPath(path)
         super(Output, self).set_shape(width, height)
 
-    def __repr__(self):
-        ''' Return the text corresponding to the SDL PR notation '''
-        comment = repr(self.comment) if self.comment else ';'
-        pos = self.scenePos()
-        return ('/* CIF OUTPUT ({x}, {y}), ({w}, {h}) */\n'
-                '{hlink}'
-                'OUTPUT {o}{comment}'.format(
-                    hlink=repr(self.text), o=str(self),
-                    x=int(pos.x()), y=int(pos.y()),
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height()), comment=comment))
-
 
 # pylint: disable=R0904
 class Decision(VerticalSymbol):
     ''' SDL DECISION Symbol '''
     _unique_followers = ['Comment']
-    _insertable_followers = ['DecisionAnswer', 'Task', 'ProcedureCall', 'Output',
-                        'Decision', 'Label']
+    _insertable_followers = ['DecisionAnswer', 'Task', 'ProcedureCall',
+                             'Output', 'Decision', 'Label']
     _terminal_followers = ['Join', 'State', 'ProcedureStop']
     common_name = 'decision'
     # Define reserved keywords for the syntax highlighter
@@ -263,24 +207,13 @@ class Decision(VerticalSymbol):
         for branch in self.branches():
             if not branch.last_branch_item.terminal_symbol:
                 return False
-        else:
-            return True
+        return True
 
     def branches(self):
         ''' Return the list of decision answers (as a generator) '''
         return (branch for branch in self.childSymbols()
                 if isinstance(branch, DecisionAnswer))
 
-    def check_syntax(self):
-        ''' Redefined function, to check only the symbol, not recursively '''
-        _, syntax_errors, ___, ____, _____ = self.parser.parseSingleElement(
-                self.common_name, self.pr(recursive=False))
-        try:
-            # LOG.error('\n'.join(syntax_errors))
-            self.scene().raise_syntax_errors(syntax_errors)
-        except AttributeError:
-            LOG.debug('raise_syntax_error raised an exception')
-
     def set_shape(self, width, height):
         ''' Define polygon points to draw the symbol '''
         path = QPainterPath()
@@ -356,30 +289,6 @@ class Decision(VerticalSymbol):
                 pass
         self.update_connections()
 
-    def pr(self, recursive=True):
-        ''' Get PR notation of a decision (possibly recursively) '''
-        comment = repr(self.comment) if self.comment else ';'
-        pos = self.scenePos()
-        result = ['/* CIF DECISION ({x}, {y}), ({w}, {h}) */\n'
-                '{hlink}'
-                'DECISION {d}{comment}'.format(
-                    hlink=repr(self.text), d=str(self),
-                    x=int(pos.x()), y=int(pos.y()),
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height()), comment=comment)]
-        if recursive:
-            for answer in self.branches():
-                if str(answer).lower().strip() == 'else':
-                    result.append(repr(answer))
-                else:
-                    result.insert(1, repr(answer))
-        result.append('ENDDECISION;')
-        return '\n'.join(result)
-
-    def __repr__(self):
-        ''' Return the PR notation for the decision and all answers '''
-        return self.pr(recursive=True)
-
 
 # pylint: disable=R0904
 class DecisionAnswer(HorizontalSymbol):
@@ -409,15 +318,6 @@ class DecisionAnswer(HorizontalSymbol):
         self.branch_entrypoint = self
         self.parser = ogParser
 
-    def check_syntax(self):
-        ''' Redefined function, to check only the symbol, not recursively '''
-        _, syntax_errors, ___, ____, _____ = self.parser.parseSingleElement(
-                self.common_name, self.pr(recursive=False))
-        try:
-            self.scene().raise_syntax_errors(syntax_errors)
-        except:
-            LOG.debug('raise_syntax_error raised an exception')
-
     def insert_symbol(self, parent, x, y):
         ''' ANSWER-specific insersion behaviour: link to connection point '''
         if not parent:
@@ -447,29 +347,6 @@ class DecisionAnswer(HorizontalSymbol):
         self.setPath(path)
         super(DecisionAnswer, self).set_shape(width, height)
 
-    def pr(self, recursive=True):
-        ''' Return the PR string for the symbol, possibly recursively '''
-        ans = str(self)
-        if ans.lower().strip() != 'else':
-            ans = '({ans})'.format(ans=ans)
-        pos = self.scenePos()
-        result = ['/* CIF ANSWER ({x}, {y}), ({w}, {h}) */\n'
-                '{hlink}'
-                '{a}:'.format(a=ans, hlink=repr(self.text),
-                    x=int(pos.x()), y=int(pos.y()),
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height()))]
-        if recursive:
-            next_symbol = self.next_aligned_symbol()
-            while next_symbol:
-                result.append(repr(next_symbol))
-                next_symbol = next_symbol.next_aligned_symbol()
-        return '\n'.join(result)
-
-    def __repr__(self):
-        ''' Return the text corresponding to the SDL PR notation '''
-        return self.pr()
-
 
 # pylint: disable=R0904
 class Join(VerticalSymbol):
@@ -510,23 +387,13 @@ class Join(VerticalSymbol):
         self.setPath(path)
         super(Join, self).set_shape(width, height)
 
-    def __repr__(self):
-        ''' Return the PR string for the join '''
-        pos = self.scenePos()
-        return('/* CIF JOIN ({x}, {y}), ({w}, {h}) */\n'
-                '{hlink}'
-                'JOIN {label};'.format(label=str(self),
-                    hlink=repr(self.text),
-                    x=int(pos.x()), y=int(pos.y()),
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height())))
-
 
 class ProcedureStop(Join):
     ''' Procedure STOP symbol - very similar to JOIN '''
     # Define reserved keywords for the syntax highlighter
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
+
     def __init__(self, parent=None, ast=None):
         if not ast:
             ast = ogAST.Terminator(defName='')
@@ -553,17 +420,6 @@ class ProcedureStop(Join):
         # call Join superclass, otherwise symbol will take Join shape
         super(Join, self).set_shape(circ, circ)
 
-    def __repr__(self):
-        ''' Return the PR string for this symbol '''
-        pos = self.scenePos()
-        return('/* CIF RETURN ({x}, {y}), ({w}, {h}) */\n'
-                '{hlink}'
-                'RETURN {val};'.format(
-                    val=str(self.text), hlink=repr(self.text),
-                    x=int(pos.x()), y=int(pos.y()),
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height())))
-
 
 # pylint: disable=R0904
 class Label(VerticalSymbol):
@@ -571,7 +427,6 @@ class Label(VerticalSymbol):
     _insertable_followers = [
             'Task', 'ProcedureCall', 'Output', 'Decision', 'Label']
     _terminal_followers = ['Join', 'State', 'ProcedureStop']
-    common_name = 'label'
     needs_parent = False
     # Define reserved keywords for the syntax highlighter
     blackbold = SDL_BLACKBOLD
@@ -613,41 +468,6 @@ class Label(VerticalSymbol):
         self.setPath(path)
         super(Label, self).set_shape(width, height)
 
-    def parse_gr(self):
-        ''' Return the PR string for a floating label (incl. transition) '''
-        pos = self.scenePos()
-        assert (not self.hasParent)
-        result = ['/* CIF LABEL ({x}, {y}), ({w}, {h}) */\n'
-                    '{hlink}'
-                    'CONNECTION {label}:'.format(
-                        label=str(self),
-                        hlink=repr(self.text),
-                        x=int(pos.x()), y=int(pos.y()),
-                        w=int(self.boundingRect().width()),
-                        h=int(self.boundingRect().height()))]
-        # Recursively parse the branch
-        next_symbol = self.next_aligned_symbol()
-        while next_symbol:
-            result.append(repr(next_symbol))
-            next_symbol = next_symbol.next_aligned_symbol()
-        result.append('/* CIF End Label */')
-        result.append('ENDCONNECTION;')
-        return '\n'.join(result)
-
-    def __repr__(self):
-        ''' Return the PR string for a label '''
-        if self.common_name == 'floating_label':
-            return self.parse_gr()
-        else:
-            # Standard label
-            pos = self.scenePos()
-            return ('/* CIF LABEL ({x}, {y}), ({w}, {h}) */\n'
-                    '{hlink}'
-                    '{label}:'.format(label=str(self), hlink=repr(self.text),
-                        x=int(pos.x()), y=int(pos.y()),
-                        w=int(self.boundingRect().width()),
-                        h=int(self.boundingRect().height())))
-
 
 # pylint: disable=R0904
 class Task(VerticalSymbol):
@@ -685,18 +505,6 @@ class Task(VerticalSymbol):
         self.setPath(path)
         super(Task, self).set_shape(width, height)
 
-    def __repr__(self):
-        ''' Return the text corresponding to the SDL PR notation '''
-        comment = repr(self.comment) if self.comment else ';'
-        pos = self.scenePos()
-        return ('/* CIF TASK ({x}, {y}), ({w}, {h}) */\n'
-                '{hlink}'
-                'TASK {t}{comment}'.format(t=str(self.text),
-                    hlink=repr(self.text),
-                    x=int(pos.x()), y=int(pos.y()),
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height()), comment=comment))
-
 
 # pylint: disable=R0904
 class ProcedureCall(VerticalSymbol):
@@ -735,18 +543,6 @@ class ProcedureCall(VerticalSymbol):
         self.setPath(path)
         super(ProcedureCall, self).set_shape(width, height)
 
-    def __repr__(self):
-        ''' Return the text corresponding to the SDL PR notation '''
-        comment = repr(self.comment) if self.comment else ';'
-        pos = self.scenePos()
-        return ('/* CIF PROCEDURECALL ({x}, {y}), ({w}, {h}) */\n'
-                '{hlink}'
-                'CALL {p}{comment}'.format(p=str(self.text),
-                    hlink=repr(self.text),
-                    x=int(pos.x()), y=int(pos.y()),
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height()), comment=comment))
-
 
 # pylint: disable=R0904
 class TextSymbol(HorizontalSymbol):
@@ -774,11 +570,10 @@ class TextSymbol(HorizontalSymbol):
         self.textbox_alignment = Qt.AlignLeft | Qt.AlignTop
         self.parser = ogParser
 
-    def update_completion_list(self):
+    def update_completion_list(self, pr_text):
         ''' When text was entered, update TASK completion list '''
         # Get AST for the symbol
-        ast, _, ___, ____, _____ = self.parser.parseSingleElement(
-                    'text_area', repr(self))
+        ast, _, _, _, _ = self.parser.parseSingleElement('text_area', pr_text)
         Task.completion_list |= {dcl for dcl in ast.variables.keys()}
 
     def set_shape(self, width, height):
@@ -802,18 +597,6 @@ class TextSymbol(HorizontalSymbol):
         self.prepareGeometryChange()
         self.set_shape(rect.width(), rect.height())
 
-    def __repr__(self):
-        ''' Return the text corresponding to the SDL PR notation '''
-        pos = self.scenePos()
-        return ('/* CIF TEXT ({x}, {y}), ({w}, {h}) */\n'
-                '{hlink}'
-                '{text}\n'
-                '/* CIF ENDTEXT */'.format(text=str(self.text),
-                    hlink=repr(self.text),
-                    x=int(pos.x()), y=int(pos.y()),
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height())))
-
 
 class ASN1Viewer(TextSymbol):
     ''' Text symbol with dedicated text highlighting set of words for ASN.1 '''
@@ -863,8 +646,8 @@ class State(VerticalSymbol):
     @property
     def allow_nesting(self):
         ''' Redefinition - must be checked according to context '''
-        result = not any(elem in str(self).lower().strip()
-                       for elem in ('-', ',', '*', ' '))
+        result = not any(elem in unicode(self).lower().strip()
+                       for elem in ('-', ',', '*'))
         return result
 
     @property
@@ -875,7 +658,7 @@ class State(VerticalSymbol):
     def double_click(self):
         ''' Catch a double click - Set nested scene '''
         for each, value in self.scene().composite_states.viewitems():
-            if str(self).lower() == str(each):
+            if unicode(self).split()[0].lower() == unicode(each):
                 self.nested_scene = value
                 break
         else:
@@ -886,24 +669,12 @@ class State(VerticalSymbol):
         ''' Set the value of the nested scene '''
         self._nested_scene = value
 
-    def update_completion_list(self):
+    def update_completion_list(self, pr_text):
         ''' When text was entered, update state completion list '''
         # Get AST for the symbol
-        ast, _, ___, ____, _____ = self.parser.parseSingleElement(
-                    'state', self.parse_gr(recursive=False))
+        ast, _, _, _, _ = self.parser.parseSingleElement('state', pr_text)
         State.completion_list |= set(ast.statelist)
 
-    def check_syntax(self):
-        ''' Redefined function, to distinguish STATE and NEXTSTATE '''
-        if self.hasParent:
-            super(State, self).check_syntax()
-        else:
-            _, syntax_err, ___, ____, _____ = self.parser.parseSingleElement(
-                    'state', self.parse_gr(recursive=False))
-            try:
-                self.scene().raise_syntax_errors(syntax_err)
-            except:
-                LOG.debug('raise_syntax_error raised an exception')
 
     def set_shape(self, width, height):
         ''' Compute the polygon to fit in width, height '''
@@ -918,77 +689,11 @@ class State(VerticalSymbol):
         self.setPath(path)
         super(State, self).set_shape(width, height)
 
-    def get_ast(self):
+    def get_ast(self, pr_text):
         ''' Redefinition of the get_ast function for the state '''
-        if self.hasParent and not [c for c in self.childSymbols()
-                if not isinstance(c, Comment)]:
-            # Terminator case
-            return super(State, self).get_ast()
-        else:
-            # State case
-            ast, _, ___, ____, terminators = self.parser.parseSingleElement(
-                'state', self.parse_gr(recursive=True))
-            return ast, terminators
-
-
-    def parse_composite_state(self):
-        ''' Return PR string corresponding to the nested part of the state '''
-        entry_points, exit_points = [], []
-        result = ['STATE {};'.format(str(self)),
-                  'SUBSTRUCTURE']
-        for each in self.nested_scene.start:
-            if str(each):
-                entry_points.append(str(each))
-        for each in self.nested_scene.returns:
-            if str(each) != 'no_name':
-                exit_points.append(str(each))
-        if entry_points:
-            result.append('in ({});'.format(','.join(entry_points)))
-        if exit_points:
-            result.append('out ({});'.format(','.join(exit_points)))
-        result.extend(self.nested_scene.get_pr_string())
-        result.append('ENDSUBSTRUCTURE;')
-        return '\n'.join(result)
-
-
-    def parse_gr(self, recursive=True):
-        ''' Parse state '''
-        comment = repr(self.comment) if self.comment else ';'
-        pos = self.scenePos()
-        if self.hasParent and not [s for s in self.childSymbols() if
-                                   not isinstance(s, Comment)]:
-            # Do not generate a new STATE when there is no need
-            # FIXME: check if childSymbol is a commment
-            return ''
-        result = ['/* CIF STATE ({x}, {y}), ({w}, {h}) */\n'
-                    '{hlink}'
-                    'STATE {state}{comment}'.format(
-                        state=str(self),
-                        hlink=repr(self.text),
-                        x=int(pos.x()), y=int(pos.y()),
-                        w=int(self.boundingRect().width()),
-                        h=int(self.boundingRect().height()), comment=comment)]
-        if recursive:
-            for i in self.childSymbols():
-                # Recursively return next symbols (inputs)
-                if isinstance(i, Input):
-                    result.append(repr(i))
-        result.append('ENDSTATE;')
-        return '\n'.join(result)
-
-    def __repr__(self):
-        ''' Return the text corresponding to the SDL PR notation '''
-        comment = repr(self.comment) if self.comment else ';'
-        pos = self.scenePos()
-        result = ['/* CIF NEXTSTATE ({x}, {y}), ({w}, {h}) */\n'
-                '{hlink}'
-                'NEXTSTATE {state}{comment}'.format(
-                    state=str(self),
-                    hlink=repr(self.text),
-                    x=int(pos.x()), y=int(pos.y()),
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height()), comment=comment)]
-        return '\n'.join(result)
+        ast, _, _, _, terminators = self.parser.parseSingleElement('state',
+                                                                   pr_text)
+        return ast, terminators
 
 
 class Process(HorizontalSymbol):
@@ -1018,6 +723,9 @@ class Process(HorizontalSymbol):
         if ast.comment:
             Comment(parent=self, ast=ast.comment)
         self.nested_scene = subscene
+        self.input_signals = ast.input_signals
+        self.output_signals = ast.output_signals
+        self.insert_symbol(None, self.x(), self.y())
 
     def insert_symbol(self, parent, x, y):
         ''' Redefinition - adds connection line to env '''
@@ -1043,49 +751,6 @@ class Process(HorizontalSymbol):
         self.setPath(path)
         super(Process, self).set_shape(width, height)
 
-    def recursive_parser(self):
-        ''' Return the full content of the process as PR string '''
-        pr_data = deque()
-        scene = self.nested_scene
-        if not scene:
-            return []
-
-        for item in chain(scene.texts, scene.procs, scene.start):
-            pr_data.append(repr(item))
-        for item in scene.floating_labels:
-            pr_data.append(item.parse_gr())
-        composite = set(scene.composite_states.keys())
-        for item in scene.states:
-            if item.is_composite():
-                try:
-                    composite.remove(str(item).lower())
-                    pr_data.appendleft(item.parse_composite_state())
-                except KeyError:
-                    pass
-            pr_data.append(item.parse_gr())
-        return list(pr_data)
-
-    def parse_gr(self, recursive=True):
-        ''' Generate PR for a process '''
-        comment = repr(self.comment) if self.comment else ';'
-        pos = self.scenePos()
-        result =['/* CIF PROCESS ({x}, {y}), ({w}, {h}) */\n'
-                 '{hlink}'
-                 'PROCESS {process}{comment}'.format(
-                        process=str(self),
-                        hlink=repr(self.text),
-                        x=int(pos.x()), y=int(pos.y()),
-                        w=int(self.boundingRect().width()),
-                        h=int(self.boundingRect().height()), comment=comment)]
-        if recursive:
-            result.append('\n'.join(self.recursive_parser()))
-        result.append('ENDPROCESS {};'.format(str(self)))
-        return '\n'.join(result)
-
-    def __repr__(self):
-        ''' Return the complete process content '''
-        return self.parse_gr(recursive=True)
-
 
 class Procedure(Process):
     ''' Procedure declaration symbol - Very similar to Process '''
@@ -1110,7 +775,7 @@ class Procedure(Process):
         if ast.comment:
             Comment(parent=self, ast=ast.comment)
         self.nested_scene = subscene
-    
+
     def insert_symbol(self, parent, x, y):
         ''' Redefinition - no connection line to env '''
         super(Process, self).insert_symbol(parent, x, y)
@@ -1130,30 +795,9 @@ class Procedure(Process):
         self.setPath(path)
         super(Process, self).set_shape(width, height)
 
-    def update_completion_list(self):
+    def update_completion_list(self, **kwargs):
         ''' When text was entered, update completion list of ProcedureCall '''
-        ProcedureCall.completion_list |= {str(self.text)}
-
-    def parse_gr(self, recursive=True):
-        ''' Generate PR for a procedure '''
-        comment = repr(self.comment) if self.comment else ';'
-        pos = self.scenePos()
-        result = ['/* CIF PROCEDURE ({x}, {y}), ({w}, {h}) */\n'
-                    '{hlink}'
-                    'PROCEDURE {proc}{comment}'.format(
-                        proc=str(self),
-                        hlink=repr(self.text),
-                        x=int(pos.x()), y=int(pos.y()),
-                        w=int(self.boundingRect().width()),
-                        h=int(self.boundingRect().height()), comment=comment)]
-        if recursive:
-            result.append('\n'.join(self.recursive_parser()))
-        result.append('ENDPROCEDURE;')
-        return '\n'.join(result)
-
-    def __repr__(self):
-        ''' Return the complete procedure branch in PR format '''
-        return self.parse_gr(recursive=True)
+        ProcedureCall.completion_list |= {unicode(self.text)}
 
 
 # pylint: disable=R0904
@@ -1177,7 +821,7 @@ class Start(HorizontalSymbol):
         ast = ast or ogAST.Start()
         self.terminal_symbol = False
         super(Start, self).__init__(parent=None,
-                                    text=ast.inputString[slice(0,-6)],
+                                    text=ast.inputString[slice(0, -6)],
                                     x=ast.pos_x, y=ast.pos_y,
                                     hyperlink=ast.hyperlink)
         self.set_shape(ast.width, ast.height)
@@ -1188,9 +832,9 @@ class Start(HorizontalSymbol):
         if ast.comment:
             Comment(parent=self, ast=ast.comment)
 
-    def __str__(self):
+    def __unicode__(self):
         ''' User cannot enter text in the START symbol - Return dummy text '''
-        return 'START'
+        return u'START'
 
     def set_shape(self, width, height):
         ''' Compute the polygon to fit in width, height '''
@@ -1199,30 +843,13 @@ class Start(HorizontalSymbol):
         self.setPath(path)
         super(Start, self).set_shape(width, height)
 
-    def __repr__(self):
-        ''' Return the text corresponding to the SDL PR notation '''
-        result = []
-        comment = repr(self.comment) if self.comment else ';'
-        pos = self.scenePos()
-        result.append('/* CIF START ({x}, {y}), ({w}, {h}) */\n'
-                'START{via}{comment}'.format(x=int(pos.x()), y=int(pos.y()),
-                    via = (' ' + str(self) + ' ')
-                    if str(self).replace('START', '') else '',
-                    w=int(self.boundingRect().width()),
-                    h=int(self.boundingRect().height()), comment=comment))
-        # Recursively return the complete branch below the start symbol
-        next_symbol = self.next_aligned_symbol()
-        while next_symbol:
-            result.append(repr(next_symbol))
-            next_symbol = next_symbol.next_aligned_symbol()
-        return '\n'.join(result)
-
 
 class ProcedureStart(Start):
     ''' Start symbol of a procedure - only shape differs from Start '''
     # Define reserved keywords for the syntax highlighter
     blackbold = SDL_BLACKBOLD
     redbold = SDL_REDBOLD
+    common_name = 'proc_start'
 
     def set_shape(self, width, height):
         ''' Compute the polygon to fit in width, height '''
@@ -1240,7 +867,8 @@ class StateStart(Start):
     ''' Composite states can have several named START symbols '''
     has_text_area = True
     is_singleton = False
+    common_name = 'state_start'
 
-    def __str__(self):
+    def __unicode__(self):
         ''' Return the state entry point '''
-        return str(self.text)
+        return unicode(self.text)
