@@ -17,14 +17,10 @@ void challenge_RI_pow(long long *a, long long *b, long long *res)
 int main()
 {
     adainit();
-    printf("[C Code] Calling RUN\n");
     challenge_run();
-    printf("[C Code] Calling RUN AGAIN....\n");
     challenge_run();
     //printf("Internal state: %lld\n", l_result);
-    printf("[C Code] Calling any_one....\n");
     challenge_any_one();
-    printf("[C Code] And finally calling RUN AGAIN....\n");
     challenge_run();
     
     return 0;
